#include "async_client.h"

Async_Client::Async_Client(boost::asio::io_service &io_service, ip::tcp::endpoint endpoint, fundata_t fundata_, void *p)
    : iosev(io_service),
      socket(iosev),
      m_ep(endpoint),
      mb_connected(0),
      mb_initialized(0),
      mb_with_reconnection(0),
      m_fundata(fundata_),
      mp_handle(p)
{
}

Async_Client::~Async_Client(){
  mp_handle = 0;
}

void Async_Client::client_async_write(char *buf, int len)
{
  boost::asio::async_write(socket,
                           boost::asio::buffer(buf, len),
                           boost::bind(&Async_Client::handle_write, this,
                                       boost::asio::placeholders::error));
}
void Async_Client::client_async_read()
{
  socket.async_read_some(boost::asio::buffer(m_origin_data_, MAX_LENGTH),
                         boost::bind(&Async_Client::handle_read, this,
                                     boost::asio::placeholders::error,
                                     boost::asio::placeholders::bytes_transferred));
}

/**
 * 返回连接状态
 * */
bool Async_Client::client_connection_status()
{
  return mb_connected;
}

/**
 * 返回初始化状态
 * */
bool Async_Client::client_initialize_status()
{
  return mb_initialized;
}

/**
 * 异步连接回调, 失败后重连
 * */
void Async_Client::handle_connect(const boost::system::error_code &error)
{
  if (!error)
  {
    LOG(INFO) << " Connection Success! " << m_ep.address().to_string() << ":" << m_ep.port();
    mb_connected = true;
    // 连接成功后开启异步读
    this->client_async_read();
  }
  else
  {
    LOG(WARNING) << boost::system::system_error(error).what() << ", start reconnect "  << m_ep.address().to_string() << ":" << m_ep.port();
    mb_connected = false;
    socket_close();
    if(mb_with_reconnection){
      socket_connect();
      usleep(1000 * 1000);
    }
  }
}

/**
 * 异步读取回调, 失败后重连
 * */
void Async_Client::handle_read(const boost::system::error_code &error,
                               size_t bytes_transferred)
{
  if (!error)
  {
    //printf("From %s Received data Len:%d\n",this->ep.address().to_string().c_str(),
    // bytes_transferred);
    if (mp_handle != 0)
    {
      // 调用外部回调处理函数后再次进入异步读
      (*m_fundata)(this->m_ep.address().to_string().c_str(), this->m_ep.port(), m_origin_data_, bytes_transferred, mp_handle);
      this->client_async_read();
    }
    else
    {
      LOG(ERROR) << "async client, handle null pointer" << m_ep.address().to_string() << ":" << m_ep.port();
    }
  }
  else
  {
    LOG(WARNING) << boost::system::system_error(error).what() << ", start reconnect " << m_ep.address().to_string() << ":" << m_ep.port();
    mb_connected = false;
    socket_close();
    if(mb_with_reconnection){
      socket_connect();
      usleep(1000 * 1000);
    }
  }
}

/**
 * 异步写入回调, 失败后重连
 * */
void Async_Client::handle_write(const boost::system::error_code &error)
{
  if (!error)
  {
  }
  else
  {
    LOG(WARNING) << boost::system::system_error(error).what() << ", start reconnect " << m_ep.address().to_string() << ":" << m_ep.port();
    mb_connected = false;
    socket_close();
    if(mb_with_reconnection){
      socket_connect();
      usleep(1000 * 1000);
    }
  }
}

/**
 * 关闭连接
 * */
void Async_Client::socket_close()
{
  socket.close();
}

/**
 * 开始连接
 * */
void Async_Client::socket_connect()
{
  socket.async_connect(m_ep,
                       boost::bind(&Async_Client::handle_connect, this, boost::asio::placeholders::error));
}

/**
 * 初始化, 参数为是否开启重连功能
 * */
bool Async_Client::initialize(bool with_reconnection)
{
  mb_with_reconnection = with_reconnection;
  socket_connect();
  mb_initialized = true;
  return mb_initialized;
}